***************************************************************************
* Calculation of poverty PPPs
*
* Program: 00_data_reshape_102_105.do
* Author : Olivier Dupriez, World Bank
* Date   : December 25, 2008
*
* Purpose: Reshape standardized survey data files to serve as input for 
*          poverty PPP calculation programs, and calculate expenditure
*          shares and variance-covariance matrix.
*          This also applies CPI and growth rates to consumption data 
*          to put all data in 2005 reference year.  
*          This is done for the 62 countries for which PPPPs will be calculated.
*
* Input  : The input data files are the standardized consumption files
*          generated according to the instructions provided in document
*          "Building a household consumption database for the calculation
*           of poverty PPPs - Technical note" (O. Dupriez, March 2007).
*          The program also requires CPI and consumption growth data
*          (extracted from WB World Development Indicators database)
*
* Note   : We generate two subsets of the 110 ICP basic headings.
*          To obtain the subset of 105 basic headings, we exclude the following:
*	     - Narcotics (47)
*            - Animal drawn vehicles (101)
*            - Maintenance and repair major durables recreation/culture (124)
*	     - Purchases by residential hhlds in the rest of the world (152)
*	     - Purchases by non-residential hhlds in the country (153)
*          To obtain the subset of 102 basic headings, we also exclude: 
*            - Rent and imputed rents (57)
*	     - Prostitution (142)
*	     - FISIM (149)
*            (codes correspond to ICP_SEQ codes)
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
**************************************************************************** 

#delimit;
clear;
set mem 400m;
set more off;
capture log close; 

local mypath  = "C:\Users\Olivier\_P4_Final\P4_Source_Files\";
local diroutp = "C:\Users\Olivier\_P4_Final\P4_Input_Files\";

* We first prepare the price data files.                           ;
* We select countries and basic headings, and adjust the parities  ;
* for India to compensate for the urban bias in ICP price survey   ;
* (details on the reason and method of adjustment is available     ;
* in a document title ...pdf)                                      ;

local file1= "`mypath'" + "_Misc\bh_icp_prices_110_final_without_OECD.dta";
use "`file1'";
keep  basic_hd p_in p_bd p_bt p_kh p_cn p_fj p_id p_la p_my p_mv p_mn 
      p_np p_pk p_ph p_lk p_th p_vn p_bj p_bf p_bi p_cm p_cv p_td p_cd 
      p_cg p_ci p_dj p_et p_ga p_gm p_gh p_gn p_ke p_ls p_mg p_mw p_ml 
      p_mr p_ma p_mz p_ne p_ng p_rw p_sn p_sl p_za p_sz p_tz p_tg p_ug
      p_ar p_bo p_br p_co p_py p_pe p_am p_az p_kz p_kg p_tj p_ye;

* We suppress 8 of the 110 basic headings;
* Narcotics, rent, animal drawn vehicles, maintenance recreation durables, prostitution,   ;
* FISIM, purchases abroad, and purchases by non residents.                                 ;
drop if inlist(basic_hd,1102311,1104111,1107141,1109231,1112211,1112611,1113111,1113112);

* We adjust parities for India to compensate for urban bias;
replace p_in=p_in*1.009849;
assert _N==102;
sort basic_hd;
gen bhd=_n;
order basic_hd bhd;
compress;
sort bhd;
save "`diroutp'bh_icp_prices_102_Final_without_OECD", replace;


use "`file1'";
keep  basic_hd p_in p_bd p_bt p_kh p_cn p_fj p_id p_la p_my p_mv p_mn 
      p_np p_pk p_ph p_lk p_th p_vn p_bj p_bf p_bi p_cm p_cv p_td p_cd 
      p_cg p_ci p_dj p_et p_ga p_gm p_gh p_gn p_ke p_ls p_mg p_mw p_ml 
      p_mr p_ma p_mz p_ne p_ng p_rw p_sn p_sl p_za p_sz p_tz p_tg p_ug
      p_ar p_bo p_br p_co p_py p_pe p_am p_az p_kz p_kg p_tj p_ye;

* We suppress 5 of the 110 basic headings;
* Narcotics, animal drawn vehicles, maintenance recreation durables,   ;
* purchases abroad, and purchases by non residents.                    ;
drop if inlist(basic_hd,1102311,1107141,1109231,1113111,1113112);

* We adjust parities for India to compensate for urban bias;
replace p_in=p_in*1.009849;
assert _N==105;
sort basic_hd;
gen bhd=_n;
order basic_hd bhd;
compress;
sort bhd;
save "`diroutp'bh_icp_prices_105_Final_without_OECD", replace;



* Now we prepare the consumption data files and variance-covariance matrixes.   ;
* For selected countries, price data are strictly urban. In such cases, we      ;
* multiply the rural households consumption data by the ratio of urban to rural ;
* poverty lines (obtained from Ravallion and Chen ("New Evidence on the         ;
* Urbanization of Global Poverty", March 2007) and consider these countries as  ;
* urban only. These countries are: Argentina, Brazil, Bolivia, Cambodia,        ;
* China, Colombia, Pakistan, Peru, and Thailand.                                ;
* For India, where price data are mainly but not strictly urban, we calculated  ;
* and applied a correction factor based on a methodology suggested by A. Deaton.;

cap program drop dataprep;

program define dataprep;

args folder dfile iso2;


local dataset = "`folder'";
local root    = "`dfile'";
local iso     = "`iso2'";  

local mypath  = "C:\Users\Olivier\_P4_Final\P4_Source_Files\";
local diroutp = "C:\Users\Olivier\_P4_Final\P4_Input_Files\";
local fileppp = "`mypath'" + "`dataset'\" + "`root'_ppp.dta";
local filelog = "`mypath'" + "`dataset'\" + "`root'_p4.log";
local fileexs = "`diroutp'" + "expsurv102_" + "`iso'";
local filevcv = "`diroutp'" + "vcv_w_102_" + "`iso'";

log using "`filelog'", replace;

* We produce the expsurv_iso.dta files (where iso=country code);
* -------------------------------------------------------------;

* We calculate the total hhld consumption (all 110 basic headings);

use "`fileppp'", clear;
replace cons_tot=cons_tot*1.43 if rururb==1 & "`iso'"=="ar";
replace cons_tot=cons_tot*1.40 if rururb==1 & "`iso'"=="bo";
replace cons_tot=cons_tot*1.55 if rururb==1 & "`iso'"=="br";
replace cons_tot=cons_tot*1.37 if rururb==1 & "`iso'"=="cn";
replace cons_tot=cons_tot*1.25 if rururb==1 & "`iso'"=="co";
replace cons_tot=cons_tot*1.06 if rururb==1 & "`iso'"=="in";
replace cons_tot=cons_tot*1.23 if rururb==1 & "`iso'"=="kh";
replace cons_tot=cons_tot*1.26 if rururb==1 & "`iso'"=="pe";
replace cons_tot=cons_tot*1.13 if rururb==1 & "`iso'"=="pk";
replace cons_tot=cons_tot*1.54 if rururb==1 & "`iso'"=="th";
collapse (sum) cons_tot, by(hid);

rename cons_tot hh_totx;
sort hid;
save "`diroutp'temp.dta", replace;


* We reshape the source file;
* We recode the icp_seq variable used to prepare the file into sequential ;
* basic heading numbers. icp_seq has 153 values, including some "virtual" ;
* basic headings. The new variable has only 102 "real" basic headings.    ;

use "`fileppp'", clear;
replace cons_tot=cons_tot*1.43 if rururb==1 & "`iso'"=="ar";
replace cons_tot=cons_tot*1.40 if rururb==1 & "`iso'"=="bo";
replace cons_tot=cons_tot*1.55 if rururb==1 & "`iso'"=="br";
replace cons_tot=cons_tot*1.37 if rururb==1 & "`iso'"=="cn";
replace cons_tot=cons_tot*1.25 if rururb==1 & "`iso'"=="co";
replace cons_tot=cons_tot*1.06 if rururb==1 & "`iso'"=="in";
replace cons_tot=cons_tot*1.23 if rururb==1 & "`iso'"=="kh";
replace cons_tot=cons_tot*1.26 if rururb==1 & "`iso'"=="pe";
replace cons_tot=cons_tot*1.13 if rururb==1 & "`iso'"=="pk";
replace cons_tot=cons_tot*1.54 if rururb==1 & "`iso'"=="th";

recode icp_seq (4=1)(5=2)(6=3)(7=4)(8=5)(10=6)(11=7)(12=8)(13=9)(14=10)
               (16=11)(17=12)(19=13)(20=14)(21=15)(22=16)(24=17)(25=18)
               (27=19)(28=20)(30=21)(31=22)(32=23)(34=24)(35=25)(36=26)
               (37=27)(39=28)(40=29)(43=30)(44=31)(45=32)(46=33) 
               (50=34)(51=35)(52=36)(54=37)(55=38)       (58=39)(60=40)
               (61=41)(63=42)(64=43)(65=44)(68=45)(69=46)(70=47)(71=48)
               (73=49)(74=50)(75=51)(76=52)(78=53)(79=54)(81=55)(82=56)
               (83=57)(87=58)(88=59)(89=60)(92=61)(93=62)(94=63)(95=64)
               (98=65)(99=66)(100=67)  (103=68)(104=69)(105=70)
               (107=71)(108=72)(109=73)(110=74)(111=75)(112=76)(114=77)
               (115=78)(116=79)(119=80)(120=81)(121=82)(123=83) 
               (126=84)(127=85)(128=86)(130=87)(131=88)(132=89)(133=90)
               (134=91)(135=92)(136=93)(137=94)(140=95)(141=96)
               (144=97)(145=98)(146=99)(147=100)       (150=101)(151=102)
               (else=.), gen(bhead);
drop if bhead==.;
drop icp_seq basic_hd ;

sort hid bhead;
rename cons_tot x;
rename wta_hh mult;
duplicates drop hid bhead, force;
reshape wide x, i(hid) j(bhead);

** We create a variable for basic headings not found in the data file;
forvalues i=1(1)102 {;
   capture confirm numeric variable x`i', exact;
   if _rc~=0 {;
      gen x`i'=0 ;
   };
};
recode x*(.=0);

* We calculate the total hhld consumption for the 102 selected items;
egen sx=rsum(x*);

sort hid;
merge hid using "`diroutp'\temp.dta"
;
tab _m;
keep if _m==3;
drop _m;

aorder;
order hid rururb stratum psu hhsize mult sx hh_totx x*;

la var x1   "Rice";
la var x2   "Other cereals, floor and other products";
la var x3   "Bread";
la var x4   "Other bakery products";
la var x5   "Pasta products";
la var x6   "Beef and veal";
la var x7   "Pork";
la var x8   "Lamb, mutton and goat";
la var x9   "Poultry";
la var x10  "Other meats and meat preparations";
la var x11  "Fresh, chilled or frozen fish and seafood";
la var x12  "Preserved or processed fish and seafood";
la var x13  "Fresh milk";
la var x14  "Preserved milk and other milk products";
la var x15  "Cheese";
la var x16  "Eggs and egg-based products";
la var x17  "Butter and margarine";
la var x18  "Other edible oil and fats";
la var x19  "Fresh or chilled fruits";
la var x20  "Frozen, preserved or processed fruit and fruit-based products";
la var x21  "Fresh or chilled vegetables other than potatoes";
la var x22  "Fresh or chilled potatoes";
la var x23  "Frozen, preserved or processed vegetables and vegetables-based products";
la var x24  "Sugar";
la var x25  "Jams, marmalades and honey";
la var x26  "Confectionery, chocolate and ice cream";
la var x27  "Food products n.e.c.";
la var x28  "Coffee, tea and cocoa";
la var x29  "Mineral waters, soft drinks, fruit and vegetable juices";
la var x30  "Spirits";
la var x31  "Wine";
la var x32  "Beer";
la var x33  "Tobacco";
* We supressed narcotics;
la var x34  "Clothing material, other articles of clothing and clothing accessories";
la var x35  "Garments";
la var x36  "Cleaning, repair and hire of clothing";
la var x37  "Shoes and other footwear";
la var x38  "Repair and hire of footwear";
* we supressed rent;
la var x39  "Maintenance and repair of the dwelling";
la var x40  "Water supply";
la var x41  "Miscellaneous services relating to the dwelling";
la var x42  "Electricity";
la var x43  "Gas";
la var x44  "Other fuels";
la var x45  "Furniture and furnishings";
la var x46  "Carpets and other floor coverings";
la var x47  "Repair of furniture, furnishings and floor coverings";
la var x48  "Household textiles";
la var x49  "Major household appliances whether electric or not";
la var x50  "Small electric household appliances";
la var x51  "Repair of household appliances";
la var x52  "Glassware, tableware and household utensils";
la var x53  "Major tools and equipment";
la var x54  "Small tools and miscellaneous accessories";
la var x55  "Non-durable household goods";
la var x56  "Domestic services";
la var x57  "Household services";
la var x58  "Pharmaceuticals products";
la var x59  "Other medical products";
la var x60  "Therapeutic appliances and equipment";
la var x61  "Medical services";
la var x62  "Dental services";
la var x63  "Paramedical services";
la var x64  "Hospital services";
la var x65  "Motor cars";
la var x66  "Motor cycles";
la var x67  "Bicycles";
* we supressed Animal drawn vehicles;
la var x68  "Fuels and lubricants for personal transport equipment";
la var x69  "Maintenance and repair of personal transport equipment";
la var x70  "Other services in respect of personal transport equipment";
la var x71  "Passenger transport by railway";
la var x72  "Passenger transport by road";
la var x73  "Passenger transport by air";
la var x74  "Passenger transport by sea and inland waterway";
la var x75  "Combined passenger transport";
la var x76  "Other purchase transport services";
la var x77  "Postal services";
la var x78  "Telephone and telefax equipment";
la var x79  "Telephone and telefax services";
la var x80  "Audio-visual, photographic and information processing equipment";
la var x81  "Recording media";
la var x82  "Repair of audio-visual, photographic and information process. equipment";
la var x83  "Major durables for outdoor and indoor recreation";
* we supressed maintenance and repair of other major durables for recreation and culture;
la var x84  "Other recreational items and equipment";
la var x85  "Garden and pets";
la var x86  "Veterinary and other services for pets";
la var x87  "Recreational and sporting services";
la var x88  "Cultural services";
la var x89  "Games of chance";
la var x90  "Newspapers, books and stationery";
la var x91  "Package holidays";
la var x92  "Education";
la var x93  "Catering services";
la var x94  "Accommodation services";
la var x95  "Hairdressing salons and personal grooming establishments";
la var x96  "Appliances, articles and products for personal care";
* we supressed prostitution;
la var x97  "Jewellery, clocks and watches";
la var x98  "Other personal effects";
la var x99  "Social protection";
la var x100 "Insurance";
* we supressed FISIM;
la var x101 "Other financial services n.e.c.";
la var x102 "Other services n.e.c.";
* we supressed purchases by residential hhlds in the rest of the world;
* we supressed purchases by non-residential hhlds in the country;
compress;
sort hid;

* We convert all values to 2005 prices using CPI data from the World Bank WDI central    ;
* database (extracted February 4, 2008) except for Djibouti,Gambia,Guinea,Tajikistan     ;
* (WDI data not available -> used World Bank Live Database).                             ;
*                                                                                        ;
* We also apply growth data to estimate consumption data in 2005.                        ;
* Data were extracted from WDI database (December 23, 2008). When available, we used     ;
* Household Final Consumption data. In a few cases, these data were not available and    ;
* we used GDP per capita growth (Burundi, Gambia, Nigeria, Sierra Leone, Togo, Fiji,     ;
* Maldives, and Nepal).                                                                  ;
*                                                                                        ;
* CPI and growth data are available in an Excel file named pppp_cpi_growth.xls           ;

gen cpi=.;
replace cpi=1.606446 if "`dataset'" == "Argentina 1996";
replace cpi=1.006412 if "`dataset'" == "Armenia 2004";
replace cpi=1.228130 if "`dataset'" == "Azerbaijan 2001";
replace cpi=1.070432 if "`dataset'" == "Bangladesh 2004";
replace cpi=1.062853 if "`dataset'" == "Benin 2003";
replace cpi=1.101351 if "`dataset'" == "Bhutan 2003";
replace cpi=1.137488 if "`dataset'" == "Bolivia 2002";
replace cpi=1.306844 if "`dataset'" == "Brazil 2002";
replace cpi=1.059891 if "`dataset'" == "Burkina_Faso 2003";
replace cpi=1.877990 if "`dataset'" == "Burundi 1998";

replace cpi=1.098106 if "`dataset'" == "Cambodia 2003";
replace cpi=1.058054 if "`dataset'" == "Cameroon 2001";
replace cpi=1.015694 if "`dataset'" == "Cape_Verde 2001";
replace cpi=1.021124 if "`dataset'" == "Chad 2003";
replace cpi=1.000000 if "`dataset'" == "China 2005";
replace cpi=1.112510 if "`dataset'" == "Colombia 2003";
replace cpi=1.000000 if "`dataset'" == "Congo_DR 2005";
replace cpi=1.000000 if "`dataset'" == "Congo_PR 2005";
replace cpi=1.089180 if "`dataset'" == "Cote_d'Ivoire 2002";
replace cpi=1.215147 if "`dataset'" == "Djibouti 1996";

replace cpi=1.116109 if "`dataset'" == "Ethiopia 2004";
replace cpi=1.096524 if "`dataset'" == "Fiji 2002";
replace cpi=1.000000 if "`dataset'" == "Gabon 2005";
replace cpi=1.178306 if "`dataset'" == "Gambia 2003";
replace cpi=0.901590 if "`dataset'" == "Ghana 2006";
replace cpi=1.743119 if "`dataset'" == "Guinea 2002";
replace cpi=1.000000 if "`dataset'" == "India 2005";
replace cpi=1.250763 if "`dataset'" == "Indonesia 2002";
replace cpi=1.149837 if "`dataset'" == "Kazakhstan 2003";
replace cpi=1.000000 if "`dataset'" == "Kenya 2005";

replace cpi=1.086804 if "`dataset'" == "Kyrgystan 2003";
replace cpi=1.183773 if "`dataset'" == "Lao_PDR 2003";
replace cpi=1.158797 if "`dataset'" == "Lesotho 2002";
replace cpi=1.544544 if "`dataset'" == "Madagascar 2001";
replace cpi=1.154103 if "`dataset'" == "Malawi 2004";
replace cpi=1.029609 if "`dataset'" == "Malaysia 2004";
replace cpi=1.032860 if "`dataset'" == "Maldives 2004";
replace cpi=0.984799 if "`dataset'" == "Mali 2006";
replace cpi=1.121256 if "`dataset'" == "Mauritania 2004";
replace cpi=1.000000 if "`dataset'" == "Mongolia 2005";

replace cpi=1.065863 if "`dataset'" == "Morocco 2001";
replace cpi=1.369498 if "`dataset'" == "Mozambique 2002";
replace cpi=1.098724 if "`dataset'" == "Nepal 2003";
replace cpi=1.000000 if "`dataset'" == "Niger 2005";
replace cpi=1.355386 if "`dataset'" == "Nigeria 2003";
replace cpi=1.245656 if "`dataset'" == "Pakistan 2001";
replace cpi=1.508711 if "`dataset'" == "Paraguay 2000";
replace cpi=1.053385 if "`dataset'" == "Peru 2003";
replace cpi=1.140598 if "`dataset'" == "Philippines 2003";
replace cpi=1.000000 if "`dataset'" == "Rwanda 2005";

replace cpi=1.044701 if "`dataset'" == "Senegal 2001";
replace cpi=1.279491 if "`dataset'" == "Sierra_Leone 2003";
replace cpi=1.280510 if "`dataset'" == "South_Africa 2000";
replace cpi=1.276811 if "`dataset'" == "Sri_Lanka 2002";
replace cpi=1.380024 if "`dataset'" == "Swaziland 2000";
replace cpi=1.133088 if "`dataset'" == "Tajikistan 2003";
replace cpi=1.194470 if "`dataset'" == "Tanzania 2000";
replace cpi=1.093698 if "`dataset'" == "Thailand 2002";
replace cpi=0.978219 if "`dataset'" == "Togo 2006";
replace cpi=1.204938 if "`dataset'" == "Uganda 2002";

replace cpi=1.082505 if "`dataset'" == "Vietnam 2004";
replace cpi=1.000000 if "`dataset'" == "Yemen 2005";
assert cpi~=.;

gen grw=.;
replace grw=1.040658 if "`dataset'" == "Argentina 1996";
replace grw=1.091007 if "`dataset'" == "Armenia 2004";
replace grw=1.221428 if "`dataset'" == "Azerbaijan 2001";
replace grw=1.108762 if "`dataset'" == "Bangladesh 2004";
replace grw=0.958660 if "`dataset'" == "Benin 2003";
replace grw=1.070408 if "`dataset'" == "Bhutan 2003";
replace grw=1.015918 if "`dataset'" == "Bolivia 2002";
replace grw=1.007318 if "`dataset'" == "Brazil 2002";
replace grw=1.023860 if "`dataset'" == "Burkina_Faso 2003";
replace grw=0.929228 if "`dataset'" == "Burundi 1998";

replace grw=1.221272 if "`dataset'" == "Cambodia 2003";
replace grw=1.073355 if "`dataset'" == "Cameroon 2001";
replace grw=1.065288 if "`dataset'" == "Cape_Verde 2001";
replace grw=1.080887 if "`dataset'" == "Chad 2003";
replace grw=1.000000 if "`dataset'" == "China 2005";
replace grw=1.082380 if "`dataset'" == "Colombia 2003";
replace grw=1.000000 if "`dataset'" == "Congo_DR 2005";
replace grw=1.000000 if "`dataset'" == "Congo_PR 2005";
replace grw=1.071548 if "`dataset'" == "Cote_d'Ivoire 2002";
replace grw=0.831968 if "`dataset'" == "Djibouti 1996";

replace grw=1.218531 if "`dataset'" == "Ethiopia 2004";
replace grw=1.096524 if "`dataset'" == "Fiji 2002";
replace grw=1.000000 if "`dataset'" == "Gabon 2005";
replace grw=1.040104 if "`dataset'" == "Gambia 2003";
replace grw=0.985317 if "`dataset'" == "Ghana 2006";
replace grw=1.025648 if "`dataset'" == "Guinea 2002";
replace grw=1.000000 if "`dataset'" == "India 2005";
replace grw=1.088718 if "`dataset'" == "Indonesia 2002";
replace grw=1.194037 if "`dataset'" == "Kazakhstan 2003";
replace grw=1.000000 if "`dataset'" == "Kenya 2005";

replace grw=1.141022 if "`dataset'" == "Kyrgystan 2003";
replace grw=1.118604 if "`dataset'" == "Lao_PDR 2003";
replace grw=1.156647 if "`dataset'" == "Lesotho 2002";
replace grw=1.059596 if "`dataset'" == "Madagascar 2001";
replace grw=0.897189 if "`dataset'" == "Malawi 2004";
replace grw=1.067439 if "`dataset'" == "Malaysia 2004";
replace grw=0.933942 if "`dataset'" == "Maldives 2004";
replace grw=0.900274 if "`dataset'" == "Mali 2006";
replace grw=1.237115 if "`dataset'" == "Mauritania 2004";
replace grw=1.000000 if "`dataset'" == "Mongolia 2005";

replace grw=1.137273 if "`dataset'" == "Morocco 2001";
replace grw=1.149173 if "`dataset'" == "Mozambique 2002";
replace grw=1.036783 if "`dataset'" == "Nepal 2003";
replace grw=1.000000 if "`dataset'" == "Niger 2005";
replace grw=1.110498 if "`dataset'" == "Nigeria 2003";
replace grw=1.141290 if "`dataset'" == "Pakistan 2001";
replace grw=0.967424 if "`dataset'" == "Paraguay 2000";
replace grw=1.055179 if "`dataset'" == "Peru 2003";
replace grw=1.065529 if "`dataset'" == "Philippines 2003";
replace grw=1.000000 if "`dataset'" == "Rwanda 2005";

replace grw=1.053749 if "`dataset'" == "Senegal 2001";
replace grw=1.064428 if "`dataset'" == "Sierra_Leone 2003";
replace grw=1.179328 if "`dataset'" == "South_Africa 2000";
replace grw=1.097510 if "`dataset'" == "Sri_Lanka 2002";
replace grw=0.875353 if "`dataset'" == "Swaziland 2000";
replace grw=1.334143 if "`dataset'" == "Tajikistan 2003";
replace grw=0.971513 if "`dataset'" == "Tanzania 2000";
replace grw=1.154982 if "`dataset'" == "Thailand 2002";
replace grw=0.987077 if "`dataset'" == "Togo 2006";
replace grw=1.040888 if "`dataset'" == "Uganda 2002";

replace grw=1.058735 if "`dataset'" == "Vietnam 2004";
replace grw=1.000000 if "`dataset'" == "Yemen 2005";
assert grw~=.;

replace hh_totx=hh_totx*cpi*grw;
replace sx=sx*cpi*grw;
forvalues i=1(1)102 {;
   replace x`i'=x`i'*cpi*grw ;
};

compress;
save "`fileexs'", replace;


**We now calculate the plutocratic budget shares and the sampling;
**variance-covariance matrix (files vcv_w_iso.dta);   
**---------------------------------------------------------------;

version 8.2;

* For some countries, we did not have information on sampling design;
* We then assume random sampling;
* Note that China datasets are synthetic (created) datasets;
svyset, clear;
if "`dataset'" == "Armenia 2004"    | "`dataset'" == "Azerbaijan 2001" 
 | "`dataset'" == "China 2005"      | "`dataset'" == "Fiji 2002"       
 | "`dataset'" == "Ghana 2006"      | "`dataset'" == "Morocco 2001" 
 | "`dataset'" == "Kazakhstan 2003" | "`dataset'" == "Kyrgystan 2003"
 | "`dataset'" == "Tajikistan 2003" {;
   svyset [pweight=mult];
 };
 else {;
   svyset [pweight=mult], strata(stratum) psu(psu);
 };


svyratio x1/sx x2/sx x3/sx x4/sx x5/sx x6/sx x7/sx x8/sx x9/sx x10/sx 
         x11/sx x12/sx x13/sx x14/sx x15/sx x16/sx x17/sx x18/sx x19/sx
         x20/sx x21/sx x22/sx x23/sx x24/sx x25/sx x26/sx x27/sx x28/sx
         x29/sx x30/sx x31/sx x32/sx x33/sx x34/sx x35/sx x36/sx x37/sx
         x38/sx x39/sx x40/sx x41/sx x42/sx x43/sx x44/sx x45/sx x46/sx
         x47/sx x48/sx x49/sx x50/sx x51/sx x52/sx x53/sx x54/sx x55/sx
         x56/sx x57/sx x58/sx x59/sx x60/sx x61/sx x62/sx x63/sx x64/sx
         x65/sx x66/sx x67/sx x68/sx x69/sx x70/sx x71/sx x72/sx x73/sx
         x74/sx x75/sx x76/sx x77/sx x78/sx x79/sx x80/sx x81/sx x82/sx
         x83/sx x84/sx x85/sx x86/sx x87/sx x88/sx x89/sx x90/sx x91/sx
         x92/sx x93/sx x94/sx x95/sx x96/sx x97/sx x98/sx x99/sx x100/sx
         x101/sx x102/sx;

matrix w_est=e(est);
matrix w_est=w_est';
matrix vcv_w=e(V_db);
drop _all;
svmat vcv_w;
svmat w_est;
rename w_est1 w_est;

log close ;
save "`filevcv'", replace;

end;

dataprep "Argentina 1996"      "arg1996"  "ar";
dataprep "Armenia 2004"        "arm2004"  "am";
dataprep "Azerbaijan 2001"     "aze2001"  "az";
dataprep "Bangladesh 2004"     "bgd2004"  "bd";
dataprep "Benin 2003"          "ben2003"  "bj";
dataprep "Bhutan 2003"         "bhu2003"  "bt";
dataprep "Bolivia 2002"        "bol2002"  "bo";
dataprep "Brazil 2002"         "bra2002"  "br";
dataprep "Burkina_Faso 2003"   "bfa2003"  "bf";
dataprep "Burundi 1998"        "bdi1998"  "bi";

dataprep "Cambodia 2003"       "khm2003"  "kh";
dataprep "Cameroon 2001"       "cmr2001"  "cm";
dataprep "Cape_Verde 2001"     "cpv2001"  "cv";
dataprep "Chad 2003"           "tcd2003"  "td";
dataprep "China 2005"          "chn2005"  "cn";
dataprep "Colombia 2003"       "col2003"  "co";
dataprep "Congo_DR 2005"       "cod2005"  "cd";
dataprep "Congo_PR 2005"       "cog2005"  "cg";
dataprep "Cote_d'Ivoire 2002"  "civ2002"  "ci";
dataprep "Djibouti 1996"       "dji1996"  "dj";

dataprep "Ethiopia 2004"       "eth2004"  "et";
dataprep "Fiji 2002"           "fji2002"  "fj";
dataprep "Gabon 2005"          "gab2005"  "ga";
dataprep "Gambia 2003"         "gmb2003"  "gm";
dataprep "Ghana 2006"          "gha2006"  "gh";
dataprep "Guinea 2002"         "gin2002"  "gn";
dataprep "India 2005"          "ind2005"  "in";
dataprep "Indonesia 2002"      "idn2002"  "id";
dataprep "Kazakhstan 2003"     "kaz2003"  "kz";
dataprep "Kenya 2005"          "ken2005"  "ke";

dataprep "Kyrgystan 2003"      "kgz2003"  "kg";
dataprep "Lao_PDR 2003"        "lao2003"  "la";
dataprep "Lesotho 2002"        "lso2002"  "ls";
dataprep "Madagascar 2001"     "mdg2001"  "mg";
dataprep "Malawi 2004"         "mwi2004"  "mw";
dataprep "Malaysia 2004"       "mys2004"  "my";
dataprep "Maldives 2004"       "mdv2004"  "mv";
dataprep "Mali 2006"           "mli2006"  "ml";
dataprep "Mauritania 2004"     "mrt2004"  "mr";
dataprep "Mongolia 2005"       "mng2005"  "mn";

dataprep "Morocco 2001"        "mar2001"  "ma";
dataprep "Mozambique 2002"     "moz2002"  "mz";
dataprep "Nepal 2003"          "npl2003"  "np";
dataprep "Niger 2005"          "ner2005"  "ne";
dataprep "Nigeria 2003"        "nga2003"  "ng";
dataprep "Pakistan 2001"       "pak2001"  "pk";
dataprep "Paraguay 2000"       "pry2000"  "py";
dataprep "Peru 2003"           "per2003"  "pe";
dataprep "Philippines 2003"    "phl2003"  "ph";
dataprep "Rwanda 2005"         "rwa2005"  "rw";

dataprep "Senegal 2001"        "sen2001"  "sn";
dataprep "Sierra_Leone 2003"   "sle2003"  "sl";
dataprep "South_Africa 2000"   "zaf2000"  "za";
dataprep "Sri_Lanka 2002"      "lka2002"  "lk";
dataprep "Swaziland 2000"      "swz2000"  "sz";
dataprep "Tajikistan 2003"     "taj2003"  "tj";
dataprep "Tanzania 2000"       "tza2000"  "tz";
dataprep "Thailand 2002"       "tha2002"  "th";
dataprep "Togo 2006"           "tgo2006"  "tg";
dataprep "Uganda 2002"         "uga2002"  "ug";

dataprep "Vietnam 2004"        "vnm2004"  "vn";
dataprep "Yemen 2005"          "yem2005"  "ye";

cap erase "`diroutp'temp.dta";
