***************************************************************************
* Calculation of poverty PPPs
*
* Program: 00_data_reshape_102_105_ALL.do
* Author : Olivier Dupriez, World Bank
* Date   : April 6, 2009
*
* Purpose: Prepares price data for the calculation of PPPs (*NOT* PPPPs)
*          This covers all countries for which poverty headcounts 
*          will be estimated.
*
* Note   : We generate two subsets of the 110 ICP basic headings.
*          To obtain the subset of 105 basic headings, we exclude the following:
*	     - Narcotics (47)
*            - Animal drawn vehicles (101)
*            - Maintenance and repair major durables recreation/culture (124)
*	     - Purchases by residential hhlds in the rest of the world (152)
*	     - Purchases by non-residential hhlds in the country (153)
*          To obtain the subset of 102 basic headings, we also exclude: 
*            - Rent and imputed rents (57)
*	     - Prostitution (142)
*	     - FISIM (149)
*            (codes correspond to ICP_SEQ codes)
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
**************************************************************************** 

#delimit;
clear;
set mem 400m;
set more off;
capture log close; 

local mypath  = "D:\_P4_Final\P4_Source_Files\";
local diroutp = "D:\_P4_Final\P4_Input_Files\";

* We first prepare the price data files.                           ;
* We select countries and basic headings, and adjust the parities  ;
* for India to compensate for the urban bias in ICP price survey   ;
* (details on the reason and method of adjustment is available     ;
* in a document title ...pdf)                                      ;

local file1= "`mypath'" + "_Misc\bh_icp_prices_110_final_without_OECD.dta";
use "`file1'";
drop bhd p_cnr p_cnu p_eg2;

* We suppress 8 of the 110 basic headings;
* Narcotics, rent, animal drawn vehicles, maintenance recreation durables, prostitution,   ;
* FISIM, purchases abroad, and purchases by non residents.                                 ;
drop if inlist(basic_hd,1102311,1104111,1107141,1109231,1112211,1112611,1113111,1113112);

* We adjust parities for India to compensate for urban bias;
replace p_in=p_in*1.009849;
assert _N==102;
sort basic_hd;
gen bhd=_n;
order basic_hd bhd;
compress;
sort bhd;
save "`diroutp'bh_icp_prices_102_Final_without_OECD_ALL", replace;


use "`file1'";
drop bhd p_cnr p_cnu p_eg2;

* We suppress 5 of the 110 basic headings;
* Narcotics, animal drawn vehicles, maintenance recreation durables,   ;
* purchases abroad, and purchases by non residents.                    ;
drop if inlist(basic_hd,1102311,1107141,1109231,1113111,1113112);

* We adjust parities for India to compensate for urban bias;
replace p_in=p_in*1.009849;
assert _N==105;
sort basic_hd;
gen bhd=_n;
order basic_hd bhd;
compress;
sort bhd;
save "`diroutp'bh_icp_prices_105_Final_without_OECD_ALL", replace;




