*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 02_cpd_index_surveys.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : Calculates CPD index using consumption weights
*             from survey data (all households) 
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
**************************************************************************** 

version 9.2
#delimit;
set logtype text;
clear;
set mem 80m;
set matsize 6350;
set more off;
cap log close; 
clear;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;

local ng=102;

* >> Enter list of countries to be included in the calculation;
* >> The first country in the list will be the one used as reference (dummy); 
*    P4s will be expressed in its national currency;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

*We count the number of countries;
global nc=0;
foreach iso of global cns {;
    global nc=$nc+1;
};
display "Number of countries is: " $nc;

* >> Enter name and location of input and log files;
local icprice  = "`dirdata'\bh_icp_prices_`ng'_Final_without_OECD.dta";
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
local vcvmat   = "vcv_w_`ng'_";
log using "`dircalc'\CPD_index_`ng'_$nc.smcl", replace;

* We check that all necessary files exist in the specified location;
foreach iso of global cns {;
    confirm file "vcv_w_`ng'_`iso'.dta";
};

* We generate temporary data files and vcv matrices;
global nn=1;
foreach ctr of global cns {;
    display "Country: `ctr'";
    use `vcvmat'`ctr';
    rename w_est w_`ctr';
    mkmat vcv_w*, mat(vcv$nn);
    gen bhd=_n;
    sort bhd;
    if $nn>1 {;
       merge bhd using temp, unique;
       tab _m; drop _m;
       sort bhd;
    };
    save temp, replace;
    global nn=$nn+1;
};

* >> Select price data to be preserved;
merge bhd using "`icprice'", 
keep (p_in p_bd p_bt p_kh p_cn p_fj p_id p_la p_my p_mv p_mn p_np p_pk p_ph p_lk p_th p_vn
      p_bj p_bf p_bi p_cm p_cv p_td p_cd p_cg p_ci p_dj p_et p_ga p_gm p_gh p_gn p_ke p_ls p_mg           
      p_mw p_ml p_mr p_ma p_mz p_ne p_ng p_rw p_sn p_sl p_za p_sz p_tz p_tg p_ug
      p_ar p_bo p_br p_co p_py p_pe p_am p_az p_kz p_kg p_tj p_ye) unique;
tab _m; 
keep if _m==3;
drop _m;

order bhd vcv_* w_* p_*;
sort bhd;
label data "price data and plutocratic shares";
save "`prices_w'", replace;

global nn=1;
foreach ctr of global cns {;
    ren p_`ctr' p$nn;
    global nn=$nn+1;
};
cap drop p_*;

global nn=1;
foreach ctr of global cns {;
    ren w_`ctr' w$nn;
    global nn=$nn+1;
};

reshape long p w, i(bhd) j(cno);
des;
gen lnp=log(p);

xi: regress lnp i.cno i.bhd;
xi: regress lnp i.cno i.bhd [aweight=w];

scalar nt=$nc*`ng';

mkmat lnp;
mkmat w;

matrix ww=diag(w);
matrix x=J(nt,1,1);
forvalues i=2(1)$nc {;
  mkmat _Icno_`i';
  matrix x=x,_Icno_`i';
  matrix drop _Icno_`i';
};

forvalues i=2(1)`ng' {;
  mkmat _Ibhd_`i';
  matrix x=x,_Ibhd_`i';
  matrix drop _Ibhd_`i';
};

drop _all;

matrix aa=syminv(x'*ww*x);
matrix b=aa*x'*ww*lnp;
matrix list b;
matrix drop ww;

matrix ppp=J($nc,1,1);
forvalues z=2(1)$nc {;
   matrix ppp[`z',1]=exp(b[`z',1]);
};
display "CPD PPPs for Global";
matrix list ppp;

**now doing the standard errors;

**sampling standard errors;

matrix zz=J(`ng',`ng',0);

forvalues w=1(1)$nc {;
   forvalues v=1(1)$nc {;
      if `v'==1 & `w'==1 {;
          matrix vv`w'=vcv1;
      };
      if `v'==1 & `w'>1 {;
          matrix vv`w'=zz;
      };
      if `v'>1 & `v'~=`w' {;
          matrix vv`w'=vv`w',zz;
      };
      if `v'>1 & `v'==`w' {;
          matrix vv`w'=vv`w',vcv`v';
      };
   };
};
forvalues i=1(1)$nc {;
   matrix drop vcv`i';
};

mat vv=vv1;
mat drop vv1;
forvalues i=2(1)$nc {;
   mat vv=vv\vv`i';
   mat drop vv`i';
   display rowsof(vv) " / " colsof(vv);
};

matrix bu=syminv(x'*x)*x'*lnp;
matrix g=lnp-x*bu;
matrix drop bu;
matrix drop lnp;
matrix g=diag(g);
matrix vs=aa*x'*g*vv*g*x*aa;
matrix drop vv;
display("Sampling standard errors");
forvalues i=1(1)$nc {;
  display sqrt(vs[`i',`i']);
};

mat d;

**failure of arbitrage standard errors;
matrix gg=g*g;
matrix drop g;
matrix ww=diag(w);
matrix drop w;
matrix vf=aa*x'*ww*gg*ww*x*aa;
matrix drop aa;
matrix drop x;
matrix drop ww;
matrix drop gg;
display("Failure of arbitrage standard errors");
forvalues i=1(1)$nc {;
  display sqrt(vf[`i',`i']);
};

matrix va=vf+vs;
display("Total standard errors");
forvalues i=1(1)$nc {;
  display sqrt(va[`i',`i']);
};

log close;

* Reminder;
display "CPD PPPs for Global";
matrix list ppp;

erase "`dirdata'\temp.dta";
