*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 03_super_index_surveys.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : Calculates EKS indexes (Tornqvist and Fisher) using 
*             consumption weights from survey data (all households) 
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
**************************************************************************** 

version 9.2
#delimit ;
clear;
set logtype text;
cap log close;
set more off;
set mem 80m;
set matsize 6500;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;

local ng=102;

* >> Enter list of countries to be included in the calculation;
* >> The first country in the list will be the one used as reference. ;
*    P4s will be expressed in its national currency;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

*We count the number of countries;
global nc=0;
foreach ctr of global cns {;
    global nc=$nc+1;
};
display "Number of countries is: " $nc;

* >> Enter name and location of input and log files;
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
log using "`dircalc'\Super_index_`ng'_$nc.smcl", replace;

* We check that all necessary files exist in the specified location;
confirm file "`dirdata'\Global_prices_w_`ng'_Final.dta";
foreach iso of global cns {;
    confirm file "vcv_w_`ng'_`iso'.dta";
};

* We generate prices and weights matrices;
global i=1;
foreach iso of global cns {;
   drop _all;
   use vcv_w_`ng'_`iso';
   mkmat vcv_w*, mat(vcv$i);
   drop _all;
   use `prices_w';
   ren p_`iso' p$i;
   ren w_`iso' w$i;
   gen lnp$i=log(p$i);
   display "Country= " "`iso'" " = number " "$i";
   mkmat lnp$i;
   mkmat p$i;
   mkmat p$i;
   mkmat w$i;
   global i=$i+1;
};

drop _all;

matrix fish=J($nc,$nc,0);
matrix torn=J($nc,$nc,0);

forvalues i=1(1)$nc {;
   forvalues j=1(1)$nc {;
      matrix fish[`i',`j']=((w`i')'*syminv(diag(p`i'))*p`j');
      matrix fish[`i',`j']=log(fish[`i',`j']);
      matrix torn[`i',`j']=0.5*(w`i'+w`j')'*(lnp`j'-lnp`i');
   };
};

matrix fish=0.5*(fish-fish');

display("Log of Tornqvist indexes");
matrix list torn;

display("Log of Fisher indexes");
matrix list fish;

* >> Identify the base country in the label;
tokenize `$cns';
display "Now making `1' the base and averaging";

matrix eks_fish=J($nc,1,0);
matrix eks_torn=J($nc,1,0);

forvalues i=1(1)$nc {;
   matrix eks_fish[`i',1]=0;
   matrix eks_torn[`i',1]=0;

   forvalues j=1(1)$nc {;
      matrix eks_torn[`i',1]=eks_torn[`i',1]+torn[1,`j']+torn[`j',`i'];
      matrix eks_fish[`i',1]=eks_fish[`i',1]+fish[1,`j']+fish[`j',`i'];
   };
};

matrix eks_fish=eks_fish/$nc;
matrix eks_torn=eks_torn/$nc;

display("Fisher EKS indexes");
matrix list eks_fish;

display("Tornqvist EKS indexes");
matrix list eks_torn;

matrix pppfish=J($nc,1,1);
forvalues z=1(1)$nc {;
   matrix pppfish[`z',1]=exp(eks_fish[`z',1]);
};
display "Fisher PPPs";
matrix list pppfish;

matrix ppptorn=J($nc,1,1);
forvalues z=1(1)$nc {;
   matrix ppptorn[`z',1]=exp(eks_torn[`z',1]);
};
display "Tornqvist PPPs";
matrix list ppptorn;

**sampling variance of the Tornqvist EKS

matrix vs_t=J($nc,1,0);

**first term;
scalar a1=0;
forvalues j=1(1)$nc {;
   matrix t1j=lnp`j'-lnp1;
   forvalues k=1(1)$nc {;
      matrix t1k=lnp`k'-lnp1;
      matrix aa=t1j'*vcv1*t1k;
      scalar a1=a1+aa[1,1];
   };
};

**second and subsequent terms;

matrix a2=J($nc,1,0);
matrix a3=J($nc,1,0);
matrix a4=J($nc,1,0);
matrix a5=J($nc,1,0);

forvalues i=1(1)$nc {;
   local a2=0;
   local a3=0;
   local a4=0;
   local a5=0;
   matrix t1i=lnp`i'-lnp1;
   matrix vit1i=vcv`i'*t1i;
   forvalues j=1(1)$nc {;
      matrix aa=t1i'*vcv1*(lnp`j'-lnp1);
      local a2=`a2'+aa[1,1];
      matrix aa=(lnp`i'-lnp`j')'*vit1i;
      local a3=`a3'+aa[1,1];
      matrix tjivi=(lnp`i'-lnp`j')'*vcv`i';
      matrix aa=t1i'*vcv`j'*t1i;
      local a5=`a5'+aa[1,1];
      forvalues k=1(1)$nc {;
         matrix aa=tjivi*(lnp`i'-lnp`k');
         local a4=`a4'+aa[1,1];
      };
   };
   matrix a2[`i',1]=2*`a2';
   matrix a3[`i',1]=2*`a3';
   matrix a4[`i',1]=`a4';
   matrix a5[`i',1]=`a5';
};

matrix fd=-1\J($nc-1,1,1);

matrix vs_t=(a1*fd+a2+a3+a4+a5)/(4*$nc*$nc);
matrix list vs_t;

**abritrage failure variance of the tornqvist;

**first running the unweighted cpd regressions in order to get the variance matrices;

**note that the lines below will have to be modified depending on the number of countries;
matrix lnp=lnp1;
mat drop lnp1;
forvalues i=2(1)$nc {;
   matrix lnp=lnp\lnp`i';
   matrix drop lnp`i';
};
matrix i1=J(`ng',1,1);
matrix i0=J(`ng',1,0);

matrix i1=J(`ng',1,1);
matrix i0=J(`ng',1,0);

forvalues w=1(1)$nc {;
   forvalues v=1(1)$nc {;
      if `v'==1 & `w'==1 {;
          matrix x`w'=i1;
      };
      if `v'==1 & `w'>1 {;
          matrix x`w'=i0;
      };
      if `v'>1 & `v'~=`w' {;
          matrix x`w'=x`w'\i0;
      };
      if `v'>1 & `v'==`w' {;
          matrix x`w'=x`w'\i1;
      };
   };
};
matrix x=x1;
forvalues i=2(1)$nc {;
   matrix x=x,x`i';
   mat drop x`i';
};
mat drop x1;

matrix bb=J(1,`ng'-1,0)\I(`ng'-1);
matrix aa=bb;
forvalues i=2(1)$nc {;
   matrix aa=aa\bb;
};
matrix x=x,aa;

matrix eu=lnp-x*syminv(x'*x)*x'*lnp;
matrix e2=diag(eu)*eu;

**these are the diagonal matrix of squared residuals for each country;

matrix om1=diag(e2[1..`ng',1]);

local ff=1;
forvalues i=2(1)$nc {;
   matrix om`i'=diag(e2[`ff'*`ng'+1..`i'*`ng',1]);
   local ff=`ff'+1;
};

matrix a1=J($nc,1,0);
matrix a2=J($nc,1,0);
matrix a3=J($nc,1,0);
matrix a4=J($nc,1,0);
matrix a5=J($nc,1,0);

forvalues i=1(1)$nc {;
   local a1=0;
   local a2=0;
   local a3=0;
   local a4=0;
   local a5=0;
   forvalues j=1(1)$nc {;
      matrix aa=(w`i'+w`j')'*om`i'*(w1-w`i');
      local a2=`a2'+2*aa[1,1];
      matrix aa=(w1-w`i')'*om1*(w1+w`j');
      local a3=`a3'-2*aa[1,1];
      matrix aa=(w1-w`i')'*om`j'*(w1-w`i');
      local a4=`a4'+aa[1,1];
      forvalues k=1(1)$nc {;
         matrix aa=(w`i'+w`j')'*om`i'*(w`i'+w`k');
         local a1=`a1'+aa[1,1];
         matrix aa=(w1+w`j')'*om1*(w1+w`k');
        local a5=`a5'+aa[1,1];
      };
   };
   matrix a1[`i',1]=`a1';
   matrix a2[`i',1]=`a2';
   matrix a3[`i',1]=`a3';
   matrix a4[`i',1]=`a4';
   matrix a5[`i',1]=`a5';
   if `i'==1 {;
      matrix a1[`i',1]=-`a1';
   };
};

matrix serr=J($nc,2,0);
matrix aerr=J($nc,2,0);
matrix terr=J($nc,2,0);
matrix serr[1,1]=0;
matrix serr[1,2]=0;

**sampling variance for the Tornqvist EKS;

matrix va_t=(a1+a2+a3+a4+a5)/(4*$nc^2);
matrix list va_t;

forvalues i=2(1)$nc {;
  matrix serr[`i',2]=sqrt(vs_t[`i',1]);
  matrix aerr[`i',2]=sqrt(va_t[`i',1]);
  matrix terr[`i',2]=sqrt(vs_t[`i',1]+va_t[`i',1]);
};

**sampling variance for the Fisher EKS;

**making the betaij's;

forvalues i=1(1)$nc {;
   forvalues j=1(1)$nc {;
      matrix r`i'`j'=J(`ng',1,0);
         forvalues n=1(1)`ng'{;
            matrix r`i'`j'[`n',1]=p`j'[`n',1]/p`i'[`n',1];
         };
      matrix aa=w`i''*r`i'`j';
      matrix beta`i'`j'=r`i'`j'/aa[1,1];
      matrix drop r`i'`j';
   };
};

matrix vs_f=J($nc,1,0);

**first term;
scalar a1=0;
forvalues j=1(1)$nc {;
   forvalues k=1(1)$nc {;
      matrix aa=beta1`j''*vcv1*beta1`k';
      scalar a1=a1+aa[1,1];
   };
};

**second and subsequent terms;

matrix a2=J($nc,1,0);
matrix a3=J($nc,1,0);
matrix a4=J($nc,1,0);
matrix a5=J($nc,1,0);

forvalues i=1(1)$nc {;
   local a2=0;
   local a3=0;
   local a4=0;
   local a5=0;
   forvalues j=1(1)$nc {;
      matrix aa=beta`i'`j''*vcv`i'*(beta`i'`i'-beta`i'1);
      local a2=`a2'-2*aa[1,1];
      matrix aa=beta1`j''*vcv1*(beta1`i'-beta11);
      local a3=`a3'+2*aa[1,1];
      matrix aa=(beta`j'`i'-beta`j'1)'*vcv`j'*(beta`j'`i'-beta`j'1);
      local a4=`a4'+aa[1,1];
      forvalues k=1(1)$nc {;
         matrix aa=beta`i'`j''*vcv`i'*beta`i'`k';
        local a5=`a5'+aa[1,1];
      };
   };
   matrix a2[`i',1]=`a2';
   matrix a3[`i',1]=`a3';
   matrix a4[`i',1]=`a4';
   matrix a5[`i',1]=`a5';
};

matrix fd=-1\J($nc-1,1,1);

matrix vs_f=(a1*fd+a2+a3+a4+a5)/(4*$nc*$nc);
matrix list vs_f;

**note that arbitrage variance for Fisher is same as for Tornqvist;
matrix va_f=va_t;

display ("Standard errors for EKS-Fisher index");

forvalues i=2(1)$nc {;
  matrix serr[`i',1]=sqrt(vs_f[`i',1]);
  matrix aerr[`i',1]=sqrt(va_f[`i',1]);
  matrix terr[`i',1]=sqrt(vs_f[`i',1]+va_f[`i',1]);
};

display("sampling error Fisher/Tornqvist");
mat list serr;
display("Arbitrage error Fisher/Tornqvist");
mat list aerr;
display("Total error Fisher/Tornqvist");
mat list terr;

log close;
