*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 206_poverty_indexes_cpd_sterrs_50plw.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : This calculates the standard and arbitrage errors 
*             for the poverty weighted cpd index,
*             with option "50 weighted poverty lines" 
* Notes     : Uses last iteration from 205_poverty_indexes_cpd_50plw.do
*           : The program is written for 29 or 102 basic headings.
*             It would need to be edited for different numbers.  
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
****************************************************************************
*
* Parameters (see " * >> " in the code below):
*    Number of basic headings
*    Location (folders) of input/output files 
*    List of countries (iso)
*    Poverty line (obtained at last iteration in 205_poverty_indexes_cpd.do)
*
****************************************************************************

version 8.2
#delimit ;
set logtype text;
cap log close;
clear;
set more off;
set matsize 6325;
set mem 90m;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;
local ng=102;

* >> Enter list of countries to be included in the calculation;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

*We count the number of countries;
global nc=0;
foreach ctr of global cns {;
    global nc=$nc+1;
};

* >> Enter location and name of input and log files;
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
local expsurv  = "expsurv`ng'_";
local pppp_cpd = "`dircalc'\PPPP_cpd_50plw_`ng'_$nc.dta";
log using "`dircalc'\PPPP_cpd_sterrs_50plw_`ng'_$nc.smcl", replace;

* We check that all necessary files exist in the specified location;
confirm file "`dirdata'\Global_prices_w_`ng'_Final.dta";
foreach iso of global cns {;
    confirm file "`expsurv'`iso'.dta";
};

* >> Enter the final poverty line (obtained at last iteration of averaging of national poverty lines);
* >> There is one poverty line per bandwidth used in the program. They are very close. We take the one for bandwidth=.5;
scalar pl0=5940.7686;
scalar lnz=log(pl0);

**Retrieving the calculated estimates of the indexes at each bandwidth;
use `pppp_cpd';
drop pppp;
reshape wide ans, i(bwn) j(cno);
mkmat ans*, mat(torn);
matrix torn=torn';
mkmat bw, mat(bws);
drop _all;

**making the logarithm of price vectors;
use `prices_w';
global cno=1;
foreach iso of global cns {;
   ren p_`iso' p$cno;
   gen lnp$cno=log(p$cno);
   mkmat lnp$cno;
   global cno=$cno+1;
};

drop _all;

**constructing the X matrix for the CPD regressions;
matrix lnp=lnp1;
forvalues i=2(1)$nc {;
   matrix lnp=lnp\lnp`i';
   mat drop lnp`i';
};
mat drop lnp1;

matrix i1=J(`ng',1,1);
matrix i0=J(`ng',1,0);

**this is with the constant term;
forvalues w=1(1)$nc {;
   forvalues v=1(1)$nc {;
      if `v'==1 & `w'==1 {;
          matrix x`w'=i1;
      };
      if `v'==1 & `w'>1 {;
          matrix x`w'=i0;
      };
      if `v'>1 & `w'==1{;
          matrix x`w'=x`w'\i1;
      };
      if `v'>1 & `w'>1 & `v'==`w' {;
          matrix x`w'=x`w'\i1;
      };
      if `v'>1 & `w'>1 & `v'~=`w' {;
          matrix x`w'=x`w'\i0;
      };
   };
};
matrix x=x1;
forvalues i=2(1)$nc {;
   matrix x=x,x`i';
   mat drop x`i';
};
mat drop x1;

matrix bb=J(1,`ng'-1,0)\I(`ng'-1);
matrix aa=bb;
forvalues i=2(1)$nc {;
   matrix aa=aa\bb;
};
matrix x=x,aa;

**making the data sets that are used to calculate the poverty weighted budget shares;
global cno=1;
foreach iso of global cns {;
   use `expsurv'`iso';
   drop if hh_totx==. | hh_totx==0;
   gen pce=hh_totx/hhsize;
   keep x* mult pce sx stratum psu;
   forvalues i=1(1)`ng' {;
      gen w`i'=x`i'/sx;
      drop x`i';
   };
   keep w* pce mult stratum psu;
   * For some countries, we did not obtain information on the sampling design (stratification);
   
   * In such cases, we assume that the sample is a simple random selection;


   svyset, clear;
   version 8.2;
   if "`iso'" == "fj" | "`iso'" == "am" | "`iso'" == "az" | "`iso'" == "kz" |
      "`iso'" == "tj" | "`iso'" == "kg" | "`iso'" == "gh" | "`iso'" == "cn" |
      "`iso'" == "ma" {;
      display "`iso'"" - No information on stratification";
      svyset [pweight=mult];
    };
    else {;
      svyset [pweight=mult], strata(stratum) psu(psu);
    };
   * This is to drop cases where all w`i' values would be missing;
   egen test=rsum(w1-w29);
   drop if test==0;
   drop test;
   save temp$cno, replace;
   global cno=$cno+1;
};

* This program makes the variance covariance matrix of the sampling estimates;
* The code below is provided for two possible number of basic headings, 29 and 102;
* For other numbers, this code must be adjusted;

cap program drop mkw_vcv;
global npr=`ng';
program define mkw_vcv;
   use temp`1', clear;
   gen lnx=log(pce);
   qui summ lnx [aw=mult];
   scalar hbw=`2'*r(sd);
   gen tau=abs(lnx-`3')/hbw;
   gen kwt=(15/(16*hbw))*(1-tau^2)^2;
   replace kwt=0 if tau >= 1;
   qui count if tau < 1;
   display "Country: " `1';
   display "No of observations in band is " r(N);
   qui summ kwt;
   forvalues i=1(1)$npr{;
      replace w`i'=w`i'*kwt;
   };

   if $npr==102 {;
      svyratio w1/kwt  w2/kwt  w3/kwt  w4/kwt  w5/kwt  w6/kwt  w7/kwt  w8/kwt  w9/kwt 
      w10/kwt w11/kwt w12/kwt w13/kwt w14/kwt w15/kwt w16/kwt w17/kwt w18/kwt w19/kwt 
      w20/kwt w21/kwt w22/kwt w23/kwt w24/kwt w25/kwt w26/kwt w27/kwt w28/kwt w29/kwt 
      w30/kwt w31/kwt w32/kwt w33/kwt w34/kwt w35/kwt w36/kwt w37/kwt w38/kwt w39/kwt
      w40/kwt w41/kwt w42/kwt w43/kwt w44/kwt w45/kwt w46/kwt w47/kwt w48/kwt w49/kwt
      w50/kwt w51/kwt w52/kwt w53/kwt w54/kwt w55/kwt w56/kwt w57/kwt w58/kwt w59/kwt
      w60/kwt w61/kwt w62/kwt w63/kwt w64/kwt w65/kwt w66/kwt w67/kwt w68/kwt w69/kwt
      w70/kwt w71/kwt w72/kwt w73/kwt w74/kwt w75/kwt w76/kwt w77/kwt w78/kwt w79/kwt
      w80/kwt w81/kwt w82/kwt w83/kwt w84/kwt w85/kwt w86/kwt w87/kwt w88/kwt w89/kwt
      w90/kwt w91/kwt w92/kwt w93/kwt w94/kwt w95/kwt w96/kwt w97/kwt w98/kwt w99/kwt
      w100/kwt w101/kwt w102/kwt ;
   };

   if $npr==29 {;
      svyratio w1/kwt w2/kwt w3/kwt w4/kwt w5/kwt w6/kwt w7/kwt w8/kwt w9/kwt 
      w10/kwt w11/kwt w12/kwt w13/kwt w14/kwt w15/kwt w16/kwt w17/kwt w18/kwt w19/kwt 
      w20/kwt w21/kwt w22/kwt w23/kwt w24/kwt w25/kwt w26/kwt w27/kwt w28/kwt w29/kwt ;
   };

   matrix w_est=e(est);
   matrix w_est=w_est';
   matrix vcv_w=e(V_db);
   drop _all;
end;

matrix zz=J(`ng',`ng',0);

matrix ses_s=J($nc,3,0);
matrix ses_f=J($nc,3,0);
matrix ses_a=J($nc,3,0);

forvalues ibw=1(1)3 {;
   scalar bw=bws[`ibw',1];
   display "Bandwidth is " bw;
   forvalues cno=1(1)$nc {;
      local lnpppp=torn[`cno',`ibw'];
      local lnpl=lnz+`lnpppp';
      mkw_vcv `cno' bw `lnpl';
      matrix w`cno'=w_est;
      matrix vcv`cno'=vcv_w;
   };
   matrix w=w1;
   forvalues j=2(1)$nc {;
      matrix w=w\w`j';
      mat drop w`j';
   };
   mat drop w1;

   matrix wd=diag(w);
   matrix aa=syminv(x'*wd*x);
   matrix b=aa*x'*wd*lnp;
   matrix drop wd;
   display("Original indexes");
   matrix tt=torn[1..$nc,`ibw'];
   matrix list tt;
   display("Confirming the PPPP indexes");
   matrix bb=b[2..3,1];
   matrix list bb;
   **sampling standard errors;

   forvalues w=1(1)$nc {;
      forvalues v=1(1)$nc {;
         if `v'==1 & `w'==1 {;
             matrix vv`w'=vcv1;
             mat drop vcv1;
         };
         if `v'==1 & `w'>1 {;
             matrix vv`w'=zz;
         };
         if `v'>1 & `v'~=`w' {;
             matrix vv`w'=vv`w',zz;
         };
         if `v'>1 & `v'==`w' {;
             matrix vv`w'=vv`w',vcv`v';
             mat drop vcv`v';
         };
      };
   };
   matrix vv=vv1;
   matrix drop vv1;
   forvalues i=2(1)$nc {;
      matrix vv=vv\vv`i';
      mat drop vv`i';
      display rowsof(vv) " / " colsof(vv);
   };

   matrix bu=syminv(x'*x)*x'*lnp;
   matrix g=lnp-x*bu;
   matrix g=diag(g);
   matrix vs=aa*x'*g*vv*g*x*aa;
   display("Sampling standard errors");
   forvalues i=1(1)$nc {;
      display sqrt(vs[`i',`i']);
      matrix ses_s[`i',`ibw']=sqrt(vs[`i',`i']);
   };
   **failure of arbitrage standard errors;
   matrix gg=g*g;
   matrix drop g;
   matrix wd=diag(w);
   matrix vf=aa*x'*wd*gg*wd*x*aa;
   matrix drop wd;
   display("Failure of arbitrage standard errors");
   forvalues i=1(1)$nc {;
      display sqrt(vf[`i',`i']);
      matrix ses_f[`i',`ibw']=sqrt(vf[`i',`i']);
   };
   matrix va=vf+vs;
   display("Total standard errors");
   forvalues i=1(1)$nc {;
      display sqrt(va[`i',`i']);
      matrix ses_a[`i',`ibw']=sqrt(va[`i',`i']);
   };
mat drop w;
mat drop gg;
mat drop aa;
};

matrix ses_s=ses_s[2..$nc,1..3];
matrix ses_f=ses_f[2..$nc,1..3];
matrix ses_a=ses_a[2..$nc,1..3];

display "Standard errors by country and bandwidth: sampling";
matrix list ses_s;
display "Standard errors by country and bandwidth: failure of arbitrage";
matrix list ses_f;
display "Standard errors by country and bandwidth: total";
matrix list ses_a;

log close;
