*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 210_poverty_indexes_tornqvist_sterrs_50plw.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : This calculates the standard and arbitrage errors 
*             for the poverty weighted tornqvist index,
*             with option "50 weighted poverty lines"  
* Notes     : Uses last iteration from 209_poverty_indexes_tornqvist_50plw.do

*           : The program is written for 29 or 102 basic headings.
*             It would need to be edited for different numbers.
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
****************************************************************************
*
* Parameters (see " * >> " in the code below):
*    Number of basic headings
*    Location (folders) of input/output files 
*    List of countries (iso)
*    Poverty line (obtained at last iteration in 209_poverty_indexes_tornqvist.do)
*
****************************************************************************

version 8.2
#delimit ;
set logtype text;
cap log close;
clear;
set matsize 6330;
set mem 100m;
set more off;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;
local ng=102;

* >> Enter list of countries to be included in the calculation;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

*We count the number of countries;
global nc=0;
foreach ctr of global cns {;
    global nc=$nc+1;
};

* >> Enter location of log file and prices/weight file to be used;
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
local pppp_tornqvist = "`dircalc'\PPPP_Tornqvist_50plw_`ng'_$nc.dta";
local expsurv  = "expsurv`ng'_";
log using "`dircalc'\PPPP_Tornqvist_sterrs_50plw_`ng'_$nc.smcl", replace;

* We check that all necessary files exist in the specified location;
confirm file "`dirdata'\Global_prices_w_`ng'_Final.dta";
foreach iso of global cns {;
    confirm file "`expsurv'`iso'.dta";
};
confirm file "`dircalc'\PPPP_Tornqvist_50plw_`ng'_$nc.dta";

* >> Enter the final poverty line (obtained at last iteration of averaging of national poverty lines);
* >> There is one poverty line per bandwidth used in the program. They are very close. We take the on for bandwidth=0.5.;
scalar pl0=5819.5542;
scalar lnz=log(pl0);

**Retrieving the calculated estimates of the indexes at each bandwidth;
use `pppp_tornqvist';
drop pppp;
reshape wide ans, i(bwn) j(cno);
mkmat ans*, mat(torn);
matrix torn=torn';
mkmat bw, mat(bws);
drop _all;

**making the logarithm of price vectors;
use `prices_w';
global cno=1;
foreach iso of global cns {;
   ren p_`iso' p$cno;
   gen lnp$cno=log(p$cno);
   mkmat lnp$cno;
   global cno=$cno+1;
};

drop _all;

**constructing the X matrix for the CPD regressions;
matrix lnp=lnp1;
forvalues i=2(1)$nc {;
   matrix lnp=lnp\lnp`i';
};

matrix i1=J(`ng',1,1);
matrix i0=J(`ng',1,0);

**this is with the constant term;
forvalues w=1(1)$nc {;
   forvalues v=1(1)$nc {;
      if `v'==1 & `w'==1 {;
          matrix x`w'=i1;
      };
      if `v'==1 & `w'>1 {;
          matrix x`w'=i0;
      };
      if `v'>1 & `w'==1{;
          matrix x`w'=x`w'\i1;
      };
      if `v'>1 & `w'>1 & `v'==`w' {;
          matrix x`w'=x`w'\i1;
      };
      if `v'>1 & `w'>1 & `v'~=`w' {;
          matrix x`w'=x`w'\i0;
      };
   };
};
matrix x=x1;
forvalues i=2(1)$nc {;
   matrix x=x,x`i';
   mat drop x`i';
};
mat drop x1;

matrix bb=J(1,`ng'-1,0)\I(`ng'-1);
matrix aa=bb;
forvalues i=2(1)$nc {;
   matrix aa=aa\bb;
};
matrix x=x,aa;

**making the data sets that are used to calculate the poverty weighted budget shares;

global cno=1;
foreach iso of global cns {;
   use `expsurv'`iso';
   drop if hh_totx==. | hh_totx==0;
   gen pce=hh_totx/hhsize;
   keep x* mult pce sx stratum psu;
   forvalues i=1(1)`ng' {;
      gen w`i'=x`i'/sx;
      drop x`i';
   };
   keep w* pce mult stratum psu;
   * For some countries, we did not obtain information on the sampling design (stratification);
   
   * In such cases, we assume that the sample is a simple random selection;

   svyset, clear;
   version 8.2;
   if "`iso'" == "fj" | "`iso'" == "am" | "`iso'" == "az" | "`iso'" == "kz" |
      "`iso'" == "tj" | "`iso'" == "kg" | "`iso'" == "gh" | "`iso'" == "cn" |
      "`iso'" == "ma" {;
      display "`iso'"" - No information on stratification";
      svyset [pweight=mult];
    };
    else {;
      svyset [pweight=mult], strata(stratum) psu(psu);
    };
   * This is to drop cases where all w`i' values would be missing;
   egen test=rsum(w1-w29);
   drop if test==0;
   drop test;
   save temp$cno, replace;
   global cno=$cno+1;
};

* This program makes the variance covariance matrix of the sampling estimates;
* The code below is provided for two possible number of basic headings, 29 and 102;
* For other numbers, this code must be adjusted;

cap program drop mkw_vcv;
global npr=`ng';
program define mkw_vcv;
   use temp`1', clear;
   gen lnx=log(pce);
   qui summ lnx [aw=mult];
   scalar hbw=`2'*r(sd);
   gen tau=abs(lnx-`3')/hbw;
   gen kwt=(15/(16*hbw))*(1-tau^2)^2;
   replace kwt=0 if tau >= 1;
   qui count if tau < 1;
   display "Country: " `1';
   display "No of observations in band is " r(N);
   qui summ kwt;
   forvalues i=1(1)$npr{;
      replace w`i'=w`i'*kwt;
   };

   if $npr==102 {;
      svyratio w1/kwt w2/kwt w3/kwt w4/kwt w5/kwt w6/kwt w7/kwt w8/kwt w9/kwt 
      w10/kwt w11/kwt w12/kwt w13/kwt w14/kwt w15/kwt w16/kwt w17/kwt w18/kwt w19/kwt 
      w20/kwt w21/kwt w22/kwt w23/kwt w24/kwt w25/kwt w26/kwt w27/kwt w28/kwt w29/kwt 
      w30/kwt w31/kwt w32/kwt w33/kwt w34/kwt w35/kwt w36/kwt w37/kwt w38/kwt w39/kwt
      w40/kwt w41/kwt w42/kwt w43/kwt w44/kwt w45/kwt w46/kwt w47/kwt w48/kwt w49/kwt
      w50/kwt w51/kwt w52/kwt w53/kwt w54/kwt w55/kwt w56/kwt w57/kwt w58/kwt w59/kwt
      w60/kwt w61/kwt w62/kwt w63/kwt w64/kwt w65/kwt w66/kwt w67/kwt w68/kwt w69/kwt
      w70/kwt w71/kwt w72/kwt w73/kwt w74/kwt w75/kwt w76/kwt w77/kwt w78/kwt w79/kwt
      w80/kwt w81/kwt w82/kwt w83/kwt w84/kwt w85/kwt w86/kwt w87/kwt w88/kwt w89/kwt
      w90/kwt w91/kwt w92/kwt w93/kwt w94/kwt w95/kwt w96/kwt w97/kwt w98/kwt w99/kwt
      w100/kwt w101/kwt w102/kwt ;
   };

   if $npr==29 {;
      svyratio w1/kwt w2/kwt w3/kwt w4/kwt w5/kwt w6/kwt w7/kwt w8/kwt w9/kwt 
      w10/kwt w11/kwt w12/kwt w13/kwt w14/kwt w15/kwt w16/kwt w17/kwt w18/kwt w19/kwt 
      w20/kwt w21/kwt w22/kwt w23/kwt w24/kwt w25/kwt w26/kwt w27/kwt w28/kwt w29/kwt ;
   };

   matrix w_est=e(est);
   matrix w_est=w_est';
   matrix vcv_w=e(V_db);
   drop _all;
end;

matrix ses_s=J($nc,3,0);
matrix ses_f=J($nc,3,0);
matrix ses_a=J($nc,3,0);

forvalues ibw=1(1)3 {;
   scalar bw=bws[`ibw',1];
   display "Bandwidth is " bw;
   forvalues cno=1(1)$nc {;
      local lnpppp=torn[`cno',`ibw'];
      local lnpl=lnz+`lnpppp';
      mkw_vcv `cno' bw `lnpl';
      matrix w`cno'=w_est;
      matrix vcv`cno'=vcv_w;
   };
   matrix b_torn=J($nc,$nc,0);
   forvalues i=1(1)$nc {;
      forvalues j=1(1)$nc {;
         matrix b_torn[`i',`j']=0.5*(w`i'+w`j')'*(lnp`j'-lnp`i');
      };
   };
   matrix eks_torn=J($nc,1,0);
   forvalues i=1(1)$nc {;
      matrix eks_torn[`i',1]=0;
      forvalues j=1(1)$nc {;
         matrix eks_torn[`i',1]=eks_torn[`i',1]+b_torn[1,`j']+b_torn[`j',`i'];
      };
   };
   matrix eks_torn=eks_torn/$nc;
   display("Matrix of pairwise Torwnvist indexes");
   matrix list b_torn;
   display "Original Tornqvist EKS indexes";
   matrix tt=torn[1..$nc,`ibw'];
   matrix list tt;
   display("Tornqvist EKS indexes");
   matrix list eks_torn;
   **sampling variance of the Tornqvist EKS;
   matrix vs_t=J($nc,1,0);
   **first term;
   scalar a1=0;
   forvalues j=1(1)$nc {;
      matrix t1j=lnp`j'-lnp1;
      forvalues k=1(1)$nc {;
         matrix t1k=lnp`k'-lnp1;
         matrix aa=t1j'*vcv1*t1k;
         scalar a1=a1+aa[1,1];
      };
   };
   **second and subsequent terms;
   matrix a2=J($nc,1,0);
   matrix a3=J($nc,1,0);
   matrix a4=J($nc,1,0);
   matrix a5=J($nc,1,0);
   forvalues i=1(1)$nc {;
      local a2=0;
      local a3=0;
      local a4=0;
      local a5=0;
      matrix t1i=lnp`i'-lnp1;
      matrix vit1i=vcv`i'*t1i;
      forvalues j=1(1)$nc {;
         matrix aa=t1i'*vcv1*(lnp`j'-lnp1);
         local a2=`a2'+aa[1,1];
         matrix aa=(lnp`i'-lnp`j')'*vit1i;
         local a3=`a3'+aa[1,1];
         matrix tjivi=(lnp`i'-lnp`j')'*vcv`i';
         matrix aa=t1i'*vcv`j'*t1i;
         local a5=`a5'+aa[1,1];
         forvalues k=1(1)$nc {;
            matrix aa=tjivi*(lnp`i'-lnp`k');
            local a4=`a4'+aa[1,1];
         };
      };
      matrix a2[`i',1]=2*`a2';
      matrix a3[`i',1]=2*`a3';
      matrix a4[`i',1]=`a4';
      matrix a5[`i',1]=`a5';
   };
   matrix fd=-1\J($nc-1,1,1);
   matrix vs_t=(a1*fd+a2+a3+a4+a5)/(4*$nc*$nc);
   matrix list vs_t;

   * Abritrage failure variance of the tornqvist;
   * First running the unweighted cpd regressions in order to get the variance matrices;

   matrix bb=J(1,`ng'-1,0)\I(`ng'-1);
   matrix aa=bb;
   forvalues i=2(1)$nc {;
      matrix aa=aa\bb;
   };
   matrix x=x,aa;
   matrix eu=lnp-x*syminv(x'*x)*x'*lnp;
   matrix e2=diag(eu)*eu;

    * These are the diagonal matrix of squared residuals for each country;
   matrix om1=diag(e2[1..`ng',1]);
   local ff=1;
   forvalues i=2(1)$nc {;
      matrix om`i'=diag(e2[`ff'*`ng'+1..`i'*`ng',1]);
      local ff=`ff'+1;
   };
   matrix a1=J($nc,1,0);
   matrix a2=J($nc,1,0);
   matrix a3=J($nc,1,0);
   matrix a4=J($nc,1,0);
   matrix a5=J($nc,1,0);
   forvalues i=1(1)$nc {;
      local a1=0;
      local a2=0;
      local a3=0;
      local a4=0;
      local a5=0;
      forvalues j=1(1)$nc {;
         matrix aa=(w`i'+w`j')'*om`i'*(w1-w`i');
         local a2=`a2'+2*aa[1,1];
         matrix aa=(w1-w`i')'*om1*(w1+w`j');
         local a3=`a3'-2*aa[1,1];
         matrix aa=(w1-w`i')'*om`j'*(w1-w`i');
         local a4=`a4'+aa[1,1];
         forvalues k=1(1)$nc {;
            matrix aa=(w`i'+w`j')'*om`i'*(w`i'+w`k');
            local a1=`a1'+aa[1,1];
            matrix aa=(w1+w`j')'*om1*(w1+w`k');
            local a5=`a5'+aa[1,1];
         };
      };
      matrix a1[`i',1]=`a1';
      matrix a2[`i',1]=`a2';
      matrix a3[`i',1]=`a3';
      matrix a4[`i',1]=`a4';
      matrix a5[`i',1]=`a5';
      if `i'==1 {;
         matrix a1[`i',1]=-`a1';
      };
   };
   matrix va_t=(a1+a2+a3+a4+a5)/(4*$nc^2);
   matrix list va_t;
   forvalues i=2(1)$nc {;
      display("sampling s.e. country `i'");
      display sqrt(vs_t[`i',1]);
      matrix ses_s[`i',`ibw']=sqrt(vs_t[`i',1]);
      display("arbitrage s.e. country `i'");
      display sqrt(va_t[`i',1]);
      matrix ses_f[`i',`ibw']=sqrt(va_t[`i',1]);
      display("total s.e. country `i'");
      display sqrt(vs_t[`i',1]+va_t[`i',1]);
      matrix ses_a[`i',`ibw']=sqrt(vs_t[`i',1]+va_t[`i',1]);
   };
   matrix drop aa;
   matrix drop bb;
};

display "Standard errors by country and bandwidth: sampling";
matrix list ses_s;

display "Standard errors by country and bandwidth: failure of arbitrage";
matrix list ses_f;

display "Standard errors by country and bandwidth: total";
matrix list ses_a;

log close;
