*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 309_poverty_indexes_tornqvist_50plw2.do
* Authors   : Angus Deaton, Princeton University
*             Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : This makes Tornqvist poverty weighted indexes for a range 
*             of bandwidths: 1.0 / 0.5 / 0.1 std dev of pc exp,
*             with option "50 weighted and doubled poverty lines"
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
****************************************************************************
*
* Parameters (see " * >> " in the code below):
*    Number of basic headings
*    Location (folders) of input/output files 
*    List of countries (iso)
*    List of countries whose national poverty lines are used (iso)
*    National poverty lines used in calculation 
*     (annual per capita, in national currency and in 2005 prices)
*    Number of iterations
*    Bandwidths (3) used for kernel weighting 
*
****************************************************************************

version 9.2
#delimit;
set logtype text;
clear;
set mem 100m;
set matsize 6330;
set more off;
cap log close; 
clear;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;

local ng=102;

* >> Enter list of countries to be included in the calculation;
* >> The first country in the list will be the one used as reference (dummy). P4s will be expressed in its national currency;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

* >> Enter sequential number of countries whose poverty lines are used as references;
global cplnb = "1 2 3 4 5 7 8 11 12 13 14 15 16 17 18 19 21 23 25 26 27 28 30 31 33 34 36 37 38 39 40 41 42 43 44 45 48
50 51 52 53 54 55 56 57 58 59 60 61 62"; 
global cplnm = "in bd bt kh cn id la mn np pk ph lk th vn bj bf cm td cg ci dj et gm gh ke ls mw ml mr ma mz ne ng rw sn 
sl tz ug ar bo br co py pe am az kz kg tj ye";

* We count the number of countries;
global nc=0;
foreach iso of global cns {;
    global nc=$nc+1;
};

* We count the number of reference poverty lines;
global nbpl=0;
foreach pln of global cplnb {;
    global nbpl=$nbpl+1;
};

* >> Enter number of iterations;
global nits=10;

* >> Enter the 3 bandwidths to be used for kernels (in units of sd of logs of pce);
matrix bws=(1\0.5\0.1);

* >> Enter name and location of input, output and log files;
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
local expsurv  = "expsurv`ng'_";
log using "`dircalc'\PPPP_Tornqvist_50plw2_`ng'_$nc.smcl", replace;
local filepppp = "`dircalc'\PPPP_Tornqvist_50plw2_`ng'_$nc.dta";

* We check that all necessary files exist in the specified location;
confirm file "`dirdata'\Global_prices_w_`ng'_Final.dta";
foreach iso of global cns {;
    confirm file "`expsurv'`iso'.dta";
};

* >> We create a matrix that will contain the national poverty lines, their ln, the P4s, the weights, and the weighted PLs. ; 
* >> These poverty lines were provided by M.Ravallion and Shaohua Chen on April 21, 2008 by email to O. Dupriez.            ;
* >> For India, the national poverty line of 4644.84 is however adjusted to me converted as an urban poverty line.          ;
* >> This adjustment is needed to adjust for the urban bias of the ICP data for India. The adjustment is as follows:        ;
* >> The national PL gives a poverty headcount of 21.97 percent of the Indian population. But to compensate for the urban   ;
* >> bias in price data, we convert prices and consumption to "all urban". For this, we have multiplied all rural           ;
* >> consumption data by a factor of 1.06. To obtain the same poverty headcount with these adjusted data, we have to use    ;
* >> a poverty line of 4900.969 Rp per capita per year.                                                                     ;
* >> The national poverty lines (per capita per year in local currency at 2005 prices) are entered in its first column.     ;

matrix zz=J($nbpl,7,0);
matrix NatPL= (4900.969\9625.496704\9502.456428\74308.2804\1270.7685024\1641709.166\1441229.538\362926.3237\8339.016809
\12592.83133\13508.85485\20443.46086\12071.63364\2181145.774\76772.57061\76428.76832\246047.2914\100079.5286\306400
\196884.8376\123688.1507\1355.013048\5576.141314\2988785.008\33223.64998\2030.211716\17321.43516\145629.6373\102779.493
\3659.078571\4120702.063\106992.2506\29593.16367\85724.69017\68177.42168\863551.46\101361.1673\344912.4\2971.526899
\4392.523688\3395.546624\2853832.036\4906088.739\1510.01513\166570.2591\1767349.106\74308.2804\9485.521147\654.312
\71432.20089),zz;

*===============================================================================;
* We multiply all poverty lines by 2;

forvalues i=1(1)$nbpl {;
   matrix NatPL[`i',1]=NatPL[`i',1]*2;
};

*===============================================================================;

forvalues i=1(1)$nbpl {;
     matrix NatPL[`i',2]=log(NatPL[`i',1]);
};

* We create the matrix used to store the poverty line calculated at each iteration, for the 3 bandwidths;
matrix NewPL=J(3,$nits,0);

* We import the starting P4s (starting values for Tornqvist PPPP, calculated with torn_approx_all);
* These were generated previously with the "Torn_Approx.do" program;
use "`dirdata'\Torn_Approx_50plw2_`ng'_$nc.dta", clear;
mkmat *, matrix(tornap);
matrix torns=tornap[1...,2];
display "Parameters used for the Tornqvist approximation:";
display "Poverty line = " tornap[1,1];

display "Number of countries = " tornap[2,1];
display "Number of basic headings = " tornap[3,1];
display "P4 approximations";
matrix list torns;

* We make the logarithm of price vectors;
use `prices_w';
global cnb=1;
foreach iso of global cns {;
   ren p_`iso' p$cnb;
   gen lnp$cnb=log(p$cnb);
   mkmat lnp$cnb;
   mkmat p$cnb;
   global cnb=$cnb+1;
};

drop _all;

global cno=1;
foreach iso of global cns {;
   use `expsurv'`iso';
   drop if hh_totx==. | sx==. | hhs==0 | hhs==.;
   gen pce=hh_totx/hhs;
   keep x* mult pce sx;
   forvalues i=1(1)`ng' {;
      gen w`i'=x`i'/sx;
      drop x`i';
   };
   keep w* pce mult;
   save temp$cno, replace;
   global cno=$cno+1;
};

cap program drop mk_wpov;
program define mk_wpov;
   use temp`1', clear;
   gen lnx=log(pce);
   qui summ lnx [aw=mult];
   scalar hbw=`2'*r(sd);
   gen tau=abs(lnx-`3')/hbw;
   gen kwt=(15/(16*hbw))*(1-tau^2)^2;
   replace kwt=0 if tau >= 1;
   qui count if tau < 1;
   display "Country: " $cno " - Bandwidth: " hhh  " - Iteration: " "$iter";
   display "No of observations in band is " r(N);
   display "Fraction of sample in band is " r(N)/_N;
   scalar nin_s=r(N);
   gen pwt=mult*kwt;
   drop mult;
   collapse w* [aweight=pwt];
   gen ct=1;
   reshape long w, i(ct) j(kgp);
   drop ct;
   sort kgp;
   mkmat w, matrix(`4');
   drop _all;
end;

matrix ans=J(5,3,0);
matrix nin=J($nc,3,0);

forvalues ibw=1(1)3 {;

   matrix torns_1=torns;
   matrix torn_ans=torns;

   matrix b=J($nc,$nc,0);
   matrix uns=J($nc,1,1);

   scalar hhh=bws[`ibw',1];
   display "Bandwith is " hhh " standard deviations of log pce";

   * We initiate the pov.lines that will be used to calculate weights for averaging poverty lines;
   * At the first iteration, we will use the national poverty lines;
   * From the 2nd iteration, these national poverty lines will be replaced with the weighted average;
   forvalues i=1(1)$nbpl {;
      matrix NatPL[`i',6]=NatPL[`i',1];
   };

   forvalues it=1(1)$nits {;

      * We extract the latest version of P4s for the countries for which the poverty lines are used;
      global lino=1;
      foreach pln of global cplnb {;
          matrix NatPL[$lino, 3]=torns_1[`pln',1];
          global lino=$lino+1;
      };

      * We convert the national poverty lines in the reference currency;
      forvalues i=1(1)$nbpl {;
           matrix NatPL[`i',4]=NatPL[`i',2]-NatPL[`i',3];
           matrix NatPL[`i',5]=exp(NatPL[`i',4]);
      };
   
     * =========================================================================================================;
     * We calculate the weighting coefficients.                                                                 ;  
     * The weights are the number of poor in each country divided by the total number of poor in all countries. ;
     * At the starting point, we estimate the nb of poor using the national poverty lines. Then we will use the ;
     * new average poverty line.                                                                                ;
     * NOTE:                                                                                                    ;
     * This section of the code is not used if we calculate PPPPs at the international poverty line obtained    ;
     * by calculating an unweighted average of national poverty lines. We however keep this part of the code    ;
     * to keep it generic. Some commands are added below to un-weight the calculation.                          ;
     * =========================================================================================================;

      global lino=1;
      display "Extrapolated population and nb of poor, and percentage of poor";
      foreach pln of global cplnm {;          
          use `expsurv'`pln', clear;
          gen pce=hh_totx/ hhsize;
          gen all=mult* hhsize;
          gen poor=mult* hhsize if pce<NatPL[$lino,6];
          collapse (sum) all poor;
          gen pcpoor=poor/all;
          mkmat *, mat(PovShr);
          mat list PovShr;
          matrix NatPL[$lino, 7]=PovShr[1,2];
          global lino=$lino+1;
      };
      global nbpoor=0;
      forvalues i=1(1)$nbpl {;
          global nbpoor=$nbpoor+NatPL[`i',7];
      };
      forvalues i=1(1)$nbpl {;
          matrix NatPL[`i',7]=NatPL[`i',7]/$nbpoor;
      };

      * We verify that the total of the weights = 1;
      global testw=0;
      forvalues i=1(1)$nbpl {;
          global testw=$testw+NatPL[`i',7];
      };
      display "Sum of weights after recalculation is: " $testw;

      * We weight these national poverty lines;
      forvalues i=1(1)$nbpl {;
           matrix NatPL[`i',8]=NatPL[`i',5]*NatPL[`i',7];
      };

      * View the new NatPL matrix;
      display 
      "NatPL: nat.pline|ln pline|lnP4s|ln nat.pline in ref.cur|nat.pline in ref.cur|int.pline in ref.cur.|weight|pline*weight"; 
      mat list NatPL;

      * We calculate the new poverty line (as a weighted average of national poverty lines);
      global npli=0;
      forvalues i=1(1)$nbpl {;
           global npli=$npli+NatPL[`i',8];
      };
      display "The new poverty line is " $npli;
      matrix NewPL[`ibw',`it']=$npli;
      mat list NewPL;

      global cno=1;
      global iter=`it';

        foreach iso of global cns {; 
                scalar ptorn=torns_1[$cno,1];
                scalar lnz=log($npli);
                scalar lnpl$cno=lnz+ptorn;
                display "Poverty line used in calculation is now: " $npli;
                mk_wpov "$cno" hhh lnpl$cno "wpov$cno";
                matrix nin[$cno,`ibw']=nin_s;
                global cno=$cno+1;
        };

      forvalues i=1(1)$nc {;
         forvalues j=1(1)$nc {;
            matrix aa=0.5*(wpov`i'+wpov`j')'*(lnp`j'-lnp`i');
            matrix b[`i',`j']=aa[1,1];
         };
      };
      matrix cc=b'*uns/$nc;
      matrix aa=b[1,1..$nc]*uns/$nc;
      matrix torns_1=cc+aa[1,1]*uns;
      display "New Tornqvist indexes";
      matrix list torns_1;
      matrix torn_ans=torn_ans,torns_1;
   
   display"Bandwidth is " hhh;
   display ("Log indexes at each iteration");
   matrix list torn_ans;

    * We calculate the value of the new poverty line in each local currency.   ;
     * This will be used to calculate the weights in the next iteration.       ;
     forvalues i=1(1)$nbpl {;
        matrix NatPL[`i',6]=$npli*exp(NatPL[`i',3]);
     };
   };

   if `ibw'==1 {;
      matrix ans=torns_1;
   };
   if `ibw' > 1 {;
      matrix ans=ans,torns_1;
   };
};

display "Reminder: list of countries is " "$cns" ;

display("Numbers in the bands in final iteration");
matrix list nin;

local cc=colsof(ans);
local rr=rowsof(ans);
matrix pppans=J(`rr',`cc',0);
forvalues i=1(1)`rr' {;
   forvalues j=1(1)`cc' {;
      matrix pppans[`i',`j']=exp(ans[`i',`j']);
   };
};

display("Final estimates of Tornqvist indexes at each bandwidth");
matrix list ans;
matrix list pppans;

svmat ans;
gen cno=_n;
reshape long ans,i(cno) j(bwn);
gen bw=1;
replace bw=0.5 if bwn==2;
replace bw=0.1 if bwn==3;

label data "Tornqvist PPPPs";
gen pppp=exp(ans);
sort bwn cno;
save "`filepppp'", replace;

log close;